@@ -1,48 +1,11 @@
-import pims
 import numpy as np
-from facemap import utils, pupil, running
+from facemap import pupil, running, utils
 from numba import vectorize,uint8,float32
 import time
 import os
 import pdb
 from scipy import io
 from scipy.ndimage import gaussian_filter
-import av
-
-
-def get_frames_pims(imall, video, cframes, cumframes):
-    nframes = cumframes[-1]
-    cframes = np.maximum(0, np.minimum(nframes-1, cframes))
-    cframes = np.arange(cframes[0], cframes[-1]+1).astype(int)
-    ivids = (cframes[np.newaxis,:] >= cumframes[1:,np.newaxis]).sum(axis=0)
-    for ii in range(len(video[0])):
-        nk = 0
-        for n in np.unique(ivids):
-            cfr = cframes[ivids==n]
-            im = np.array(video[n][ii][cfr[0]-cumframes[n]:cfr[-1]-cumframes[n]+1])[:,:,:,0]
-            imall[ii][nk:nk+im.shape[0]] = im
-            nk += im.shape[0]
-    if nk < imall[0].shape[0]:
-        for ii,im in enumerate(imall):
-            imall[ii] = im[:nk].copy()
-
-def get_frames(imall, containers, cframes, cumframes):
-    cframes = np.arange(cframes[0], min(cframes[-1], cumframes[-1])).astype(int)
-    ivids = (cframes[np.newaxis,:] >= cumframes[1:,np.newaxis]).sum(axis=0)
-    for ii in range(len(containers[0])):
-        nk = 0
-        for n in np.unique(ivids):
-            cfr = cframes[ivids==n]
-            k=0
-            for frame in containers[n][ii].decode(video=0):
-                imall[ii][nk] = frame.to_ndarray(format='rgb24')[:,:,0]
-                k+=1
-                nk+=1
-                if k==cfr[-1]-cfr[0]+1:
-                    break
-    if nk < imall[0].shape[0]:
-        for ii,im in enumerate(imall):
-            imall[ii] = im[:nk].copy()
 
 def binned_inds(Ly, Lx, sbin):
     Lyb = np.zeros((len(Ly),), np.int32)
@@ -75,7 +38,7 @@ def imall_init(nfr, Ly, Lx):
 
 def subsampled_mean(containers, cumframes, Ly, Lx, sbin=3):
     # grab up to 2000 frames to average over for mean
-    # v is a list of containerss loaded with av
+    # v is a list of containers loaded with av
     # cumframes are the cumulative frames across videos
     # Ly, Lx are the sizes of the videos
     # sbin is the spatial binning
@@ -86,7 +49,6 @@ def subsampled_mean(containers, cumframes, Ly, Lx, sbin=3):
     nsegs = int(np.floor(nf / nt0))
     # what times to sample
     tf = np.floor(np.linspace(0, nframes - nt0, nsegs)).astype(int)
-
     # binned Ly and Lx and their relative inds in concatenated movies
     Lyb, Lxb, ir = binned_inds(Ly, Lx, sbin)
     imall = imall_init(nt0, Ly, Lx)
@@ -94,9 +56,10 @@ def subsampled_mean(containers, cumframes, Ly, Lx, sbin=3):
     avgframe  = np.zeros(((Lyb * Lxb).sum(),), np.float32)
     avgmotion = np.zeros(((Lyb * Lxb).sum(),), np.float32)
     ns = 0
+    
     for n in range(nsegs):
         t = tf[n]
-        get_frames_pims(imall, containers, np.arange(t,t+nt0), cumframes)
+        utils.get_frames(imall, containers, np.arange(t,t+nt0), cumframes)
         # bin
         for n,im in enumerate(imall):
             imbin = spatial_bin(im, sbin, Lyb[n], Lxb[n])
@@ -105,7 +68,6 @@ def subsampled_mean(containers, cumframes, Ly, Lx, sbin=3):
             imbin = np.abs(np.diff(imbin, axis=0))
             avgmotion[ir[n]] += imbin.mean(axis=0)
         ns+=1
-
     avgframe /= float(ns)
     avgmotion /= float(ns)
     avgframe0 = []
@@ -160,10 +122,9 @@ def compute_SVD(containers, cumframes, Ly, Lx, avgmotion, ncomps=500, sbin=3, ro
 
     ns = 0
     for n in range(nsegs):
-        tic=time.time()
         img = imall_init(nt0, Ly, Lx)
         t = tf[n]
-        get_frames_pims(img, containers, np.arange(t,t+nt0), cumframes)
+        utils.get_frames(img, containers, np.arange(t,t+nt0), cumframes)
         if fullSVD:
             imall = np.zeros((img[0].shape[0]-1, (Lyb*Lxb).sum()), np.float32)
         for ii,im in enumerate(img):
@@ -286,7 +247,7 @@ def process_ROIs(containers, cumframes, Ly, Lx, avgmotion, U, sbin=3, tic=None,
     for n in range(nsegs):
         t += nt1
         img = imall_init(nt0, Ly, Lx)
-        get_frames_pims(img, containers, np.arange(t, min(cumframes[-1],t+nt0)), cumframes)
+        utils.get_frames(img, containers, np.arange(t,t+nt0), cumframes)
         nt1 = img[0].shape[0]
         # compute pupil
         if len(pupind)>0:
@@ -301,7 +262,6 @@ def process_ROIs(containers, cumframes, Ly, Lx, avgmotion, U, sbin=3, tic=None,
                 pups[k]['area'][t:t+nt1] = area
                 pups[k]['axdir'][t:t+nt1,:,:] = axdir
                 pups[k]['axlen'][t:t+nt1,:] = axlen
-                #print(area)
                 k+=1
 
         if len(blind)>0:
@@ -416,30 +376,26 @@ def save(proc, savepath=None):
         del d2
     return savename
 
+
 def run(filenames, parent=None, proc=None, savepath=None):
+    '''
+    Parameters
+    ----------
+    filenames : list of names of video(s) to get
+    '''
     ''' uses filenames and processes fullSVD if no roi's specified '''
     ''' parent is from GUI '''
     ''' proc can be a saved ROI file from GUI '''
     ''' savepath is the folder in which to save _proc.npy '''
-    print('processing videos')
+    start = time.time()
+    print('Processing videos...')
     # grab files
-    Lys = []
-    Lxs = []
     rois=None
     sy,sx=0,0
     if parent is not None:
         filenames = parent.filenames
-        video = parent.video
-        containers = []
-        for fs in filenames:
-            cs = []
-            for f in fs:
-                cs.append(av.open(f))
-                cs[-1].streams.video[0].thread_type = 'AUTO'
-            containers.append(cs)
+        _, _, _, containers = utils.get_frame_details(filenames)
         cumframes = parent.cumframes
-        nframes = parent.nframes
-        iframes = parent.iframes
         sbin = parent.sbin
         rois = utils.roi_to_dict(parent.ROIs, parent.rROI)
         Ly = parent.Ly
@@ -449,39 +405,9 @@ def run(filenames, parent=None, proc=None, savepath=None):
         sy = parent.sy
         sx = parent.sx
     else:
-        video=[]
-        containers = []
-        nframes = 0
-        iframes = []
-        cumframes = [0]
-        k=0
-        for fs in filenames:
-            vs = []
-            cs = []
-            for f in fs:
-                try:
-                    vs.append(pims.Video(f))
-                except:
-                    vs.append(pims.PyAVReaderIndexed(f))
-                cs.append(av.open(f))
-                cs[-1].streams.video[0].thread_type = 'AUTO'
-            video.append(vs)
-            containers.append(cs)
-            iframes.append(len(video[-1][0]))
-            cumframes.append(cumframes[-1] + len(video[-1][0]))
-            nframes += len(video[-1][0])
-            if k==0:
-                Ly = []
-                Lx = []
-                for vs in video[-1]:
-                    fshape = vs.frame_shape
-                    Ly.append(fshape[0])
-                    Lx.append(fshape[1])
-            k+=1
-        iframes = np.array(iframes).astype(int)
-        cumframes = np.array(cumframes).astype(int)
+        cumframes, Ly, Lx, containers = utils.get_frame_details(filenames)
         if proc is None:
-            sbin = 4
+            sbin = 1
             fullSVD = True
             save_mat = False
             rois=None
@@ -491,7 +417,7 @@ def run(filenames, parent=None, proc=None, savepath=None):
             save_mat = proc['save_mat']
             rois = proc['rois']
             sy = proc['sy']
-            sx = proc['sx']
+            sx = proc['sx']   
 
     Lybin, Lxbin, iinds = binned_inds(Ly, Lx, sbin)
     LYbin,LXbin,sybin,sxbin = utils.video_placement(Lybin, Lxbin)
@@ -506,13 +432,9 @@ def run(filenames, parent=None, proc=None, savepath=None):
                                             np.floor(r['xrange'][-1])/sbin).astype(int)
                 nroi+=1
 
-    isRGB = True
-    #if len(frame_shape) > 2:
-    #    isRGB = True
-
     tic = time.time()
     # compute average frame and average motion across videos (binned by sbin)
-    avgframe, avgmotion = subsampled_mean(video, cumframes, Ly, Lx, sbin)
+    avgframe, avgmotion = subsampled_mean(containers, cumframes, Ly, Lx, sbin)  
     avgframe_reshape = utils.multivideo_reshape(np.hstack(avgframe)[:,np.newaxis],
                                           LYbin,LXbin,sybin,sxbin,Lybin,Lxbin,iinds)
     avgframe_reshape = np.squeeze(avgframe_reshape)
@@ -524,7 +446,7 @@ def run(filenames, parent=None, proc=None, savepath=None):
     ncomps = 500
     if fullSVD or nroi>0:
         # compute SVD from frames subsampled across videos and return spatial components
-        U = compute_SVD(video, cumframes, Ly, Lx, avgmotion, ncomps, sbin, rois, fullSVD)
+        U = compute_SVD(containers, cumframes, Ly, Lx, avgmotion, ncomps, sbin, rois, fullSVD)
         print('computed subsampled SVD at %0.2fs'%(time.time() - tic))
         U_reshape = U.copy()
         if fullSVD:
@@ -540,13 +462,13 @@ def run(filenames, parent=None, proc=None, savepath=None):
     else:
         U = []
         U_reshape = []
-
+   
     # project U onto all movie frames
     # and compute pupil (if selected)
-    V, M, pups, blinks, runs = process_ROIs(video, cumframes, Ly, Lx, avgmotion, U, sbin, tic, rois, fullSVD)
+    V, M, pups, blinks, runs = process_ROIs(containers, cumframes, Ly, Lx, avgmotion, U, sbin, tic, rois, fullSVD)
 
     # smooth pupil and blinks and running
-    print('smoothing ...')
+    print('Smoothing ...')
     for p in pups:
         if 'area' in p:
             p['area_smooth'],_ = pupil.smooth(p['area'].copy())
@@ -557,9 +479,8 @@ def run(filenames, parent=None, proc=None, savepath=None):
         b,_ = pupil.smooth(b.copy())
 
     print('computed projection at %0.2fs'%(time.time() - tic))
-
     proc = {
-            'filenames': filenames, 'save_path': savepath, 'iframes': iframes, 'Ly': Ly, 'Lx': Lx,
+            'filenames': filenames, 'save_path': savepath, 'Ly': Ly, 'Lx': Lx,
             'sbin': sbin, 'fullSVD': fullSVD, 'save_mat': save_mat,
             'Lybin': Lybin, 'Lxbin': Lxbin,
             'sybin': sybin, 'sxbin': sxbin, 'LYbin': LYbin, 'LXbin': LXbin,
@@ -570,7 +491,10 @@ def run(filenames, parent=None, proc=None, savepath=None):
             'pupil': pups, 'running': runs, 'blink': blinks, 'rois': rois,
             'sy': sy, 'sx': sx
             } 
-
+    
     # save processing
     savename = save(proc, savepath)
-    return savename
+    utils.close_videos(containers)
+    print('run time %0.2fs'%(time.time() - start))
+
+    return savename
\ No newline at end of file
